<#-- Import utils -->
<#import "../../application/docbook+xml/common_module_traversal_utils.ftl" as utils>
<#-- Import common macros and functions -->
<#import "../../application/docbook+xml/macros_common_general.ftl" as com>
<#compress>
<#-- Initialize common variables -->
<@com.initializeMainVariables/>
<#if _subject.submissionType?has_content && _subject.submissionType?matches("EU_PPP_.*")>
	<@com.initiRelevanceForPPP relevance/>
</#if>
<#-- Root entity -->
<#assign rootEntity = entity.root>
<#-- Utility list to remember tarversed entities -->
<#assign traveresedEntityKeyList = [rootEntity.documentKey]>
<#-- Print CSV header row -->
<@printCsvHeadersRow />
<#-- Utility list to remember entities yet to be traversed -->
<#assign toTravereseEntityList = []>
<#-- Traverse the main entity -->
<@traverseEntity rootEntity />
<#-- Go throught the found referenced entities and traverse them. NOTE: the list can grow further as entities can reference further entities. Unclear whether this works well in FreeMarker -->
<#-- FreeMarker does not support do-while loops as such do this workaround with maxLoop -->

<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->
<#assign maxLoop=5>
   <#list 1..maxLoop as x>           
      <#-- Remove documents that have been already traversed -->
      <#assign toTravereseEntityList = utils.removeDocumentWitKey(toTravereseEntityList, traveresedEntityKeyList) >
      <#if !(toTravereseEntityList?has_content)>
        <#-- Stop looping if there is nothing to traverse -->
        <#break>
      </#if>
      <#-- Travers each entity from the list -->
      <#list toTravereseEntityList as entityItem>
        <#assign traveresedEntityKeyList= traveresedEntityKeyList + [entityItem.documentKey]>
          <@traverseEntity entityItem />
      </#list>          
    </#list>

<#macro traverseEntity entityDoc>
  <#if utils.isCompositeEntityDocument(entityDoc)>
    <@traverseCompositeEntity entityDoc />
  <#else>
    <@traverseSimpleEntity entityDoc />
  </#if>
</#macro>

<#macro traverseSimpleEntity entityDoc>
  <@traverseDoc entityDoc entityDoc 0 />
</#macro>

<#macro traverseCompositeEntity entityDoc>
  <#local tableOfContents=iuclid.localizeTreeFor(entityDoc.documentType, entityDoc.submissionType, entityDoc.documentKey, utils.getInheritedTemplates(entityDoc), utils.getRelatedCategories(entityDoc), utils.getRelatedMixtures(entityDoc))>
  <@traverseToc tableOfContents entityDoc 0 />
</#macro>

<#macro traverseToc sectionNode entityDocument level=0>
  <#if sectionNode?has_content>
    <#-- iterate through the section documents under this section if there are any -->
    <#if sectionNode.content?has_content>
      <#list sectionNode.content as doc>
        <#-- traverse through the document's blocks and fields aka elements -->
        <@traverseDoc doc entityDocument level+1 sectionNode />
      </#list>
    </#if>
    <#-- iterate through the child sections of the node and do a recursive call -->
    <#if sectionNode?children?has_content>
      <#list sectionNode?children as child> 
        <@traverseToc child entityDocument level+1 />    
      </#list>
    </#if>
  </#if>
</#macro>

<#macro traverseDoc docElementNode entityDocument level=0 sectionNode="" isDocument=true>
  <#if docElementNode?has_content>
    <#-- Only document level -->
    <#if isDocument>
      <@printLiteratureReferencesForDocument docElementNode entityDocument sectionNode />
    </#if>
    <#-- Check if the element node is a reference field (single or multiple) and remember referenced entities for later traversal in toTravereseEntityList -->
    <#assign toTravereseEntityList = utils.checkAndRememberReferencedEntity(docElementNode, toTravereseEntityList, traveresedEntityKeyList) />
    <#-- iterate through the child structure elements of this document element if there are any and do a recursive call -->
    <#if docElementNode?children?has_content>
      <#list docElementNode?children as child>
        <@traverseDoc child entityDocument level+1 sectionNode false/>
      </#list>
    </#if>
  </#if>
</#macro>

<#macro printLiteratureReferencesForDocument sectionDoc entityDocument sectionNode>
 
  <#local sectionName = "" />
  <#if sectionNode?has_content>
    <#if sectionNode.number?has_content>
      <#local sectionName = sectionNode.number + " ">
    </#if>
    <#local sectionName = sectionName + sectionNode.title>
  </#if>

  <#local sectionDocType = sectionDoc.documentType>

  <#local sectionDocSubType = sectionDoc.documentSubType>

  <#if !utils.isEntityDocument(sectionDoc)>
    <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
  </#if>

  <#-- The web URL pointing to the document -->
  <#local docUrl=utils.getDocumentUrl(sectionDoc) />
  <#assign listOfReferences = getReferencedLitRefEntitiesFromDoc(sectionDoc) >
    <#if listOfReferences?has_content>  
    	<#list listOfReferences as literatureReferenceDpList>
      		<#local literatureReference=literatureReferenceDpList[0]/>
      		<#local dataPr=literatureReferenceDpList[1]/>
        	<#if literatureReference?has_content>
          		<@tableOutput literatureReference sectionName sectionDoc docUrl entityDocument dataPr/> 
        	</#if>
      	</#list>								
    </#if>     
</#macro>

<#macro tableOutput literatureReference sectionName sectionDoc docUrl entityDocument dataPr>
	<#local literatureAuthor = "">
	<#local literatureTitle = "">
	<#local literatureYear = "">
	<#local literatureReportNo = "">
	<#local literatureReportDate = "">
	<#local literatureSource = "">
	<#local literatureSponsor = "">
	<#local literatureType = "">
	<#local literatureLab = "">
	<#local literatureStudyNo = "">
	<#local literatureRemarks = "">
	<#if literatureReference.GeneralInfo.CompanyOwner?has_content>
		<#local literatureSponsor = literatureReference.GeneralInfo.CompanyOwner />
	</#if>
  	<#if literatureReference.GeneralInfo.Author?has_content>
		<#local literatureAuthor = literatureReference.GeneralInfo.Author>
	</#if>
	<#if literatureReference.GeneralInfo.Name?has_content>
		<#local literatureTitle = literatureReference.GeneralInfo.Name>
	</#if>
	<#if literatureReference.GeneralInfo.ReferenceYear?has_content>
		<#local literatureYear><@com.number literatureReference.GeneralInfo.ReferenceYear/></#local>
	</#if>
	<#if literatureReference.GeneralInfo.ReportNo?has_content>
		<#local literatureReportNo = literatureReference.GeneralInfo.ReportNo>
	</#if>
	<#if literatureReference.GeneralInfo.ReportDate?has_content>
		<#local literatureReportDate = literatureReference.GeneralInfo.ReportDate>
	</#if>
	<#if literatureReference.GeneralInfo.Source?has_content>
		<#local literatureSource = literatureReference.GeneralInfo.Source>
	</#if>
	<#if literatureReference.GeneralInfo.LiteratureType?has_content>
		<#local literatureType><@com.picklist literatureReference.GeneralInfo.LiteratureType/></#local>
	</#if>
	<#if literatureReference.GeneralInfo.TestLab?has_content>
		<#local literatureLab = literatureReference.GeneralInfo.TestLab>
	</#if>
	<#if literatureReference.GeneralInfo.CompanyOwnerStudyNo?has_content>
		<#local literatureStudyNo = literatureReference.GeneralInfo.CompanyOwnerStudyNo>
	</#if>
	<#if literatureReference.GeneralInfo.Remarks?has_content>
		<#local literatureRemarks = literatureReference.GeneralInfo.Remarks>
	</#if>
      
	<#local sectionDocTypePath = sectionDoc.documentType + "." + sectionDoc.documentSubType>
	<#local sectionDocType = sectionDoc.documentType>

	<#if sectionDoc.hasElement("AdministrativeData.DataProtection")>
		<#local flag=sectionDoc.AdministrativeData.DataProtection/>
  	<#elseif sectionDoc.hasElement("AdministrativeDataSummary.DataProtection")>
		<#local flag=sectionDoc.AdministrativeDataSummary.DataProtection/>
	<#else>
		<#local flag=''>
		<#--missing: in some FLEXIBLE RECORDS: Suppliers, Sites, GhS-->
		<#--  <#local flagPath="DataProtection"/>-->
	</#if>
	
	<#if sectionDoc.hasElement("MaterialsAndMethods.GLPComplianceStatement") && sectionDoc.MaterialsAndMethods.GLPComplianceStatement?has_content>
		<#local glpData = utils.getPicklistFieldAsText(sectionDoc.MaterialsAndMethods.GLPComplianceStatement)/>
	<#else>
		<#local glpData = 'no GLP information available'/>
	</#if>
      
	<#if dataPr?has_content && !(dataPr?is_string) >
		<#local dpValue><@com.picklist picklistValue=dataPr printDescription=false printRemarks=false/></#local>
		<#local dpJust><@com.text dataPr.remarks/></#local>      			
	<#elseif dataProtection?has_content>
		<#local dpValue = dataProtection/>
		<#local dpJust = dataProtection/>
	<#else>
		<#local dpValue	= 'no'/>
		<#local dpJust = ''/>
	</#if>
	
	<#if pppRelevant??>
		<#local previouslyUsed=getChangeLogStatus(sectionDoc, entityDocument)/>
	</#if>
	<#local litStudyIds=getLitIds(literatureReference)/>

<@utils.csvValue entityDocument.documentType/>,<@utils.csvValue entityDocument.name/>,<@utils.csvValue sectionName/>,"=HYPERLINK(""${docUrl}"",""${sectionDoc.name}"")",<@utils.csvValue glpData/>,<#if flag?has_content && flag.confidentiality?has_content>yes<#else>no</#if>,<#if flag?has_content && flag.justification?has_content><@utils.csvValue flag.justification/></#if>,<@utils.csvValue dpValue/>,<@utils.csvValue dpJust/>,<#if pppRelevant??><@utils.csvValue previouslyUsed/>,</#if><@utils.csvValue literatureType/>,<@utils.csvValue literatureAuthor/>,<@utils.csvValue literatureTitle/>,<@utils.csvValue literatureYear/>,<@utils.csvValue literatureReportNo/>,<@utils.csvValue literatureReportDate/>,<@utils.csvValue literatureStudyNo/>,<@utils.csvValue literatureSource/>,<@utils.csvValue literatureSponsor/>,<@utils.csvValue literatureLab/>,<@utils.csvValue literatureRemarks/>,<@utils.csvValue litStudyIds/>  
</#macro>

<#function getReferencedLitRefEntitiesFromDoc docElementNode>
    <#local listOfLiteratureReferences = [] />

    <#if docElementNode?node_type == "document_reference">
      <#local listOfLiteratureReferences = checkAndAddLiteraturReferenceWithDP(docElementNode, docElementNode, listOfLiteratureReferences) />
      <#return listOfLiteratureReferences />
    </#if>

    <#if docElementNode?node_type == "document_references">
      <#list docElementNode as item>
        <#local listOfLiteratureReferences = checkAndAddLiteraturReferenceWithDP(item, docElementNode, listOfLiteratureReferences) />
      </#list>
      <#return listOfLiteratureReferences />
    </#if>
    
    <#if docElementNode?children?has_content>   
      <#list docElementNode?children as child>
      <#local listOfLiteratureReferences = listOfLiteratureReferences + getReferencedLitRefEntitiesFromDoc(child)  />
      </#list>
    </#if>
    <#return listOfLiteratureReferences/>
</#function>

<#function checkAndAddLiteraturReference docKey listOfReferencedEntities>
  <#local doc = iuclid.getDocumentForKey(docKey) />
  <#if doc?has_content && doc.documentType == "LITERATURE">
      <#return utils.addDocumentToSequenceAsUnique(doc, listOfReferencedEntities)>
  </#if>
  <#return listOfReferencedEntities>
</#function>

<#-- This function is a new version of the one above, in order to include both the lit reference and data protection claim -->
<#function checkAndAddLiteraturReferenceWithDP docKey docNode listOfReferencedEntities>
	<#local doc = iuclid.getDocumentForKey(docKey) />
	<#if doc?has_content && doc.documentType == "LITERATURE">
	  	<#list listOfReferencedEntities as docDpList>
	  		<#local document=docDpList[0]/>
	    	<#if document.documentKey == doc.documentKey>
	      		<#return listOfReferencedEntities>
	    	</#if>
	  	</#list>
	  	
	  	<#-- get DP -->
	  	<#local dataProt=""/>
	  	<#local docKeyParents=docNode?ancestors/>
	  	<#if docKeyParents?has_content>
	  		<#local docKeyParent = docKeyParents[0]/>
  			<#if docKeyParent.hasElement("DataProtectionClaimed")>
  				<#local dataProt=docKeyParent.DataProtectionClaimed/>
  			<#else>
  				<#local dataProt="N/A"/>
  			</#if>  
  		</#if> 
  		
  		<#-- add -->
  		<#local newLitEntry = [doc, dataProt]/>
		<#return listOfReferencedEntities + [newLitEntry]>
  	</#if>
  	<#return listOfReferencedEntities>
</#function>

<#macro printCsvHeadersRow>
"Entity Type","Entity Name","Section Name","Document Name","GLP","Confidentiality Claim","Confidentiality Justification","Data Protection Claimed","Data Protection Justification",<#if pppRelevant??>"Previously Used",</#if>"Literature Type","Literature Author","Literature Title","Literature Year","Literature Report No","Literature Report Date","Literature Study Number","Literature Source","Literature Study Sponsor","Literature Test Facility","Literature Remarks","Literature Other Study Identifier(s)"
</#macro>
</#compress>

<#-- This function checks if the document exists in the change log and retrieves the status -->
<#function getChangeLogStatus study entity>

	<#local changeLogFlag="no"/>
	<#local changeLogEntryList=[]/>
	<#local changeLogs = iuclid.getSectionDocumentsForParentKey(entity.documentKey, "FLEXIBLE_RECORD", "ChangeLog") />
	<#list changeLogs as changeLog>
		<#list changeLog.ChangeLog.ChangeLogEntries as changeLogEntry>
			<#local changeLogDoc=iuclid.getDocumentForKey(changeLogEntry.LinkToDocument)/>

			<#if changeLogDoc?has_content>
				<#if study.documentKey.uuid==changeLogDoc.documentKey.uuid>
					<#local changeLogStatus><@com.picklist changeLogEntry.Status/></#local>
					<#if changeLogStatus?has_content>
						<#if !(changeLogStatus?starts_with("new"))>
							<#local changeLogFlag="yes"/>
						</#if>
						<#-- <#local changeLogStatusList = changeLogStatusList + [changeLogStatus]/> -->
						<#local changeLogEntryList = changeLogEntryList + [changeLogEntry]/>
					</#if>
				</#if>
			</#if>
		</#list>
	</#list>
	
	<#local changeLogStatusMessage>
		<@compress single_line=true>
		${changeLogFlag}
		<#if changeLogEntryList?has_content>
			<#list changeLogEntryList as clEntry><#if clEntry.Status?has_content> (<@com.picklist clEntry.Status/>)</#if><#if clEntry.Remark?has_content> - <@com.text clEntry.Remark/></#if><#if clEntry?has_next> | </#if></#list>
		</#if>
		</@compress>
	</#local>
	
	<#return changeLogStatusMessage/>
</#function>

<#function getLitIds ref>
	<#local litIds>
		<@compress single_line=true>
			<#list ref.GeneralInfo.StudyIdentifiers as studyId>
				<#if studyId.StudyIDType?has_content><@com.picklist studyId.StudyIDType/>: </#if><@com.text studyId.StudyID/><#if studyId.Remarks?has_content> (<@com.text studyId.Remarks/>)</#if>
				<#if studyId?has_next>|</#if>
			</#list>
		</@compress>
	</#local>
	<#return litIds/>
</#function>
